/*
 * Java
 *
 * Copyright 2015-2022 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.util;

import ej.bon.Constants;
import ej.sni.SNI;

public class Device {

	private static final String BUFFER_SIZE_CONSTANT_NAME = "ej.device.buffer.size";
	private static final int BUFFER_SIZE = Constants.getInt(BUFFER_SIZE_CONSTANT_NAME);

	private Device() {
		// private constructor
	}

	public static String getArchitecture() {
		byte[] buffer = new byte[BUFFER_SIZE];
		if (!DeviceNatives.getArchitecture(buffer, buffer.length)) {
			throw new IllegalArgumentException(); // wrong implementation
		}

		return SNI.toJavaString(buffer);
	}

	public static byte[] getId() {
		byte[] buffer = new byte[BUFFER_SIZE];
		int idSize = DeviceNatives.getId(buffer, buffer.length);
		if (idSize <= 0) {
			throw new IllegalArgumentException(); // wrong implementation
		}

		byte[] id = new byte[idSize];
		System.arraycopy(buffer, 0, id, 0, idSize);
		return id;
	}

	public static String getVersion() {
		byte[] buffer = new byte[BUFFER_SIZE];
		DeviceNatives.getVersion(buffer);
		return SNI.toJavaString(buffer);
	}

	public static void reboot() {
		DeviceNatives.reboot();
	}

	public static void shutdown() {
		DeviceNatives.shutdown();
	}

}
